def chunk_phrases(text):
    import spacy
    nlp = spacy.load("en_core_web_md")
    doc = nlp(text)
    phrases = []

    for chunk in doc.noun_chunks:
        phrases.append(chunk.text)

    for token in doc:
        if token.pos_ in {"VERB", "NOUN"}:
            phrases.append(token.lemma_)

    return list(set(phrases))



from keywordx import KeywordExtractor

ke = KeywordExtractor()
text = "Tomorrow I have a work meeting at 5pm in Bangalore."
keywords = ["meeting", "time", "place", "date"]

result = ke.extract(text, keywords)
print(result)


import numpy as np
from sklearn.metrics.pairwise import cosine_similarity

def score_matches(kw_emb, cand_embs, phrases, idf_vectorizer=None, idf_map=None, baseline_emb=None):
    sims = cosine_similarity([kw_emb], cand_embs)[0]

    if idf_vectorizer is not None:
        tfidf = idf_vectorizer.transform(phrases).toarray()
        tfidf_weights = tfidf.max(axis=1)
    elif idf_map is not None:
        tfidf_weights = np.array([idf_map.get(p, 1.0) for p in phrases])
    else:
        tfidf_weights = np.ones(len(phrases))

    tfidf_weights = tfidf_weights / (tfidf_weights.max() + 1e-6)

    final_scores = 0.7 * sims + 0.3 * tfidf_weights
    return final_scores



import dateparser
from datetime import datetime

def extract_structured(text, ref_date=None):
    from spacy import load
    nlp = load("en_core_web_md")
    doc = nlp(text)
    res = []

    for ent in doc.ents:
        if ent.label_ in {"DATE","TIME","MONEY","CARDINAL","LOC","GPE"}:
            res.append({"type": ent.label_, "text": ent.text, "span": (ent.start_char, ent.end_char)})

    if ref_date is None:
        ref_date = datetime.now()

    d = dateparser.parse(text, settings={'RELATIVE_BASE': ref_date})
    if d:
        res.append({"type":"PARSED_DATE","text": text, "value": d.isoformat()})

    return res
